@@ -14,7 +14,7 @@ arc.strength.test = function(network, data, test, alpha, B, debug = FALSE) {
     parents =
       network$nodes[[arc[2]]]$parents[network$nodes[[arc[2]]]$parents != arc[1]]
 
-    a = conditional.test(arc[1], arc[2], parents, data = data, test = test,
+    a = indep.test(arc[1], arc[2], parents, data = data, test = test,
           B = B, alpha = alpha)
 
     if (debug) {
