@@ -11,11 +11,11 @@ choose.direction.test = function(x, arc, data, test, alpha, B, debug = FALSE) {
   # does not depend on them so they are largely irrelevant.
 
   parents1 = parents.backend(x$arcs, arc[2], undirected = TRUE)
-  a1 = conditional.test(arc[1], arc[2], parents1[parents1 != arc[1]],
+  a1 = indep.test(arc[1], arc[2], parents1[parents1 != arc[1]],
         data = data, test = test, B = B, alpha = alpha)
 
   parents2 = parents.backend(x$arcs, arc[1], undirected = TRUE)
-  a2 = conditional.test(arc[2], arc[1], parents2[parents2 != arc[2]],
+  a2 = indep.test(arc[2], arc[1], parents2[parents2 != arc[2]],
         data = data, test = test, B = B, alpha = alpha)
 
   if (debug) {
