@@ -16,8 +16,10 @@ nparams = function(x, data, debug = FALSE) {
     # nparams is unknown for partially directed graphs.
     if (is.pdag(x$arcs, names(x$nodes)))
       stop("the graph is only partially directed.")
+    # check which type of data we are dealing with.
+    type = data.type(data)
 
-    if (is.data.discrete(data))
+    if (type %in% c("factor", "ordered", "mixed-do"))
       return(nparams.discrete(x, data, real = TRUE, debug = debug))
     else
       return(nparams.gaussian(x, debug = debug))
@@ -100,7 +102,7 @@ blacklist = function(x) {
 }#BLACKLIST
 
 # reconstruct the equivalence class of a network.
-cpdag = function(x, moral = TRUE, debug = FALSE) {
+cpdag = function(x, moral = FALSE, debug = FALSE) {
 
   # check x's class.
   check.bn(x)
@@ -142,7 +144,7 @@ cextend = function(x, strict = TRUE, debug = FALSE) {
 }#CEXTEND
 
 # report v-structures in the network.
-vstructs = function(x, arcs = FALSE, moral = TRUE, debug = FALSE) {
+vstructs = function(x, arcs = FALSE, moral = FALSE, debug = FALSE) {
 
   # check x's class.
   check.bn(x)
