@@ -4,10 +4,10 @@
 SEXP nparams_dnet(SEXP graph, SEXP data, SEXP real, SEXP debug) {
 
 int i = 0, j = 0, nnodes = 0;
-int *index = NULL, *r = LOGICAL(real), *debuglevel = LOGICAL(debug);
-double node_params = 0;
-double *res = NULL, *nlevels = NULL;
-SEXP nodes, node_data, parents, try, result;
+int *index = NULL, r = isTRUE(real), debuglevel = isTRUE(debug);
+double node_params = 0, all_params = 0;
+double *nlevels = NULL;
+SEXP nodes, node_data, parents, try;
 
   /* get nodes' number and data. */
   node_data = getListElement(graph, "nodes");
@@ -19,11 +19,6 @@ SEXP nodes, node_data, parents, try, result;
   for (i = 0; i < nnodes; i++)
     nlevels[i] = NLEVELS2(data, i);
 
-  /* allocate and initialize the return value. */
-  PROTECT(result = allocVector(REALSXP, 1));
-  res = REAL(result);
-  res[0] = 0;
-
   /* for each node... */
   for (i = 0; i < nnodes; i++) {
 
@@ -42,31 +37,29 @@ SEXP nodes, node_data, parents, try, result;
     UNPROTECT(1);
 
     /* multiply by the number of free parameters. */
-    if (*r > 0)
+    if (r > 0)
       node_params *= nlevels[i] - 1;
     else
       node_params *= nlevels[i];
 
-    if (*debuglevel > 0)
+    if (debuglevel > 0)
       Rprintf("* node %s has %.0lf parameter(s).\n", NODE(i), node_params);
 
     /* update the return value. */
-    res[0] += node_params;
+    all_params += node_params;
 
   }/*FOR*/
 
-  UNPROTECT(1);
-
-  return result;
+  return ScalarReal(all_params);
 
 }/*NPARAMS_DNET*/
 
 /* get the number of parameters of a single node (discrete case). */
 SEXP nparams_dnode(SEXP graph, SEXP node, SEXP data, SEXP real) {
 
-int i = 0, j = 0, length_nodes = 0, *r = LOGICAL(real);
-double  *nlevels = NULL;
-SEXP temp, names, result;
+int i = 0, j = 0, length_nodes = 0, r = isTRUE(real);
+double nlevels = 1;
+SEXP temp, names;
 
   /* get the entry for the parents of the node.*/
   temp = getListElement(graph, "nodes");
@@ -78,11 +71,6 @@ SEXP temp, names, result;
   names = getAttrib(data, R_NamesSymbol);
   length_nodes = length(temp);
 
-  /* allocate and initialize the result. */
-  PROTECT(result = allocVector(REALSXP, 1));
-  nlevels = REAL(result);
-  *nlevels = 1;
-
   /* sum (multiply, actually) up the levels. */
   for (i = 0; i < length(names); i++) {
 
@@ -91,7 +79,7 @@ SEXP temp, names, result;
       /* this is a parent. */
       if (!strcmp(CHAR(STRING_ELT(names, i)), CHAR(STRING_ELT(temp, j)))) {
 
-        *nlevels *= NLEVELS2(data, i);
+        nlevels *= NLEVELS2(data, i);
 
       }/*THEN*/
 
@@ -100,30 +88,24 @@ SEXP temp, names, result;
     /* this is the node. */
     if (!strcmp(CHAR(STRING_ELT(names, i)), CHAR(STRING_ELT(node, 0)))) {
 
-      *nlevels *= NLEVELS2(data, i) - 1 * (*r);
+      nlevels *= NLEVELS2(data, i) - 1 * r;
 
     }/*THEN*/
 
   }/*FOR*/
 
-  UNPROTECT(1);
-
-return result;
+  return ScalarReal(nlevels);
 
 }/*NPARAMS_DNODE*/
 
 /* get the number of parameters of the whole network (continuous case). */
 SEXP nparams_gnet(SEXP graph, SEXP debug) {
 
-int i = 0, node_params = 0, *res = NULL, *debuglevel = LOGICAL(debug);
-SEXP result, nodes = R_NilValue, temp = getListElement(graph, "nodes");
+int i = 0, node_params = 0, debuglevel = isTRUE(debug);
+double all_params = 0;
+SEXP nodes = R_NilValue, temp = getListElement(graph, "nodes");
 
-  /* allocate and initialize the result. */
-  PROTECT(result = allocVector(INTSXP, 1));
-  res = INTEGER(result);
-  res[0] = 0;
-
-  if (*debuglevel > 0)
+  if (debuglevel > 0)
     nodes = getAttrib(temp, R_NamesSymbol);
 
   /* add one parameter for each regressor, which means one for each
@@ -132,17 +114,15 @@ SEXP result, nodes = R_NilValue, temp = getListElement(graph, "nodes");
 
     node_params = length(getListElement(VECTOR_ELT(temp, i), "parents")) + 1;
 
-    if (*debuglevel > 0)
+    if (debuglevel > 0)
       Rprintf("* node %s has %d parameter(s).\n", NODE(i), node_params);
 
     /* update the return value. */
-    res[0] += node_params;
+    all_params += node_params;
 
   }/*FOR*/
 
-  UNPROTECT(1);
-
-  return(result);
+  return ScalarReal(all_params);
 
 }/*NPARAMS_GNET*/
 
@@ -150,17 +130,13 @@ SEXP result, nodes = R_NilValue, temp = getListElement(graph, "nodes");
 SEXP nparams_gnode(SEXP graph, SEXP node) {
 
 char *name = (char *)CHAR(STRING_ELT(node, 0));
-SEXP temp, result;
+SEXP temp;
 
   temp = getListElement(graph, "nodes");
   temp = getListElement(temp, name);
   temp = getListElement(temp, "parents");
 
-  PROTECT(result = allocVector(INTSXP, 1));
-  INT(result) = length(temp) + 1;
-  UNPROTECT(1);
-
-  return result;
+  return ScalarReal(length(temp) + 1);
 
 }/*NPARAMS_GNODE*/
 
@@ -169,7 +145,7 @@ SEXP nbr2arcs(SEXP nbr) {
 
 int i = 0, j = 0, k = 0, narcs = 0;
 int length_names = 0;
-SEXP arcs, dimnames, colnames, temp, names;
+SEXP arcs, dimnames, temp, names;
 
   /* get the names of the nodes. */
   names = getAttrib(nbr, R_NamesSymbol);
@@ -188,10 +164,7 @@ SEXP arcs, dimnames, colnames, temp, names;
 
   /* allocate colnames. */
   PROTECT(dimnames = allocVector(VECSXP, 2));
-  PROTECT(colnames = allocVector(STRSXP, 2));
-  SET_STRING_ELT(colnames, 0, mkChar("from"));
-  SET_STRING_ELT(colnames, 1, mkChar("to"));
-  SET_VECTOR_ELT(dimnames, 1, colnames);
+  SET_VECTOR_ELT(dimnames, 1, mkStringVec(2, "from", "to"));
 
   /* if there are no arcs, return an empty arc set. */
   if (narcs == 0) {
@@ -232,9 +205,9 @@ SEXP arcs, dimnames, colnames, temp, names;
 
   }/*FOR*/
 
-  UNPROTECT(3);
+  UNPROTECT(2);
 
-return arcs;
+  return arcs;
 
 }/*NBR2ARCS*/
 
@@ -243,22 +216,15 @@ SEXP all_equal(SEXP target, SEXP current) {
 
 int nnodes = 0,  narcs = 0;
 int *t = NULL, *c = NULL;
-SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash, result;
+SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
 
   /* get the node set of each network. */
   tnodes = getAttrib(getListElement(target, "nodes"), R_NamesSymbol);
   cnodes = getAttrib(getListElement(current, "nodes"), R_NamesSymbol);
 
   /* first check: node sets must have the same size. */
-  if (length(tnodes) != length(cnodes)) {
-
-    PROTECT(result = allocVector(STRSXP, 1));
-    SET_STRING_ELT(result, 0, mkChar("Different number of nodes"));
-    UNPROTECT(1);
-
-    return result;
-
-  }/*THEN*/
+  if (length(tnodes) != length(cnodes))
+    return mkString("Different number of nodes");
 
   /* store for future use. */
   nnodes = length(tnodes);
@@ -276,11 +242,9 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash, result;
 
     if (c[i] != i + 1) {
 
-      PROTECT(result = allocVector(STRSXP, 1));
-      SET_STRING_ELT(result, 0, mkChar("Different node sets"));
-      UNPROTECT(2);
+      UNPROTECT(1);
 
-      return result;
+      return mkString("Different node sets");
 
     }/*THEN*/
 
@@ -293,15 +257,8 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash, result;
   carcs = getListElement(current, "arcs");
 
   /* third check: arc sets must have the same size. */
-  if (length(tarcs) != length(carcs)) {
-
-    PROTECT(result = allocVector(STRSXP, 1));
-    SET_STRING_ELT(result, 0, mkChar("Different number of directed/undirected arcs"));
-    UNPROTECT(1);
-
-    return result;
-
-  }/*THEN*/
+  if (length(tarcs) != length(carcs))
+    return mkString("Different number of directed/undirected arcs");
 
   /* store for future use. */
   narcs = length(tarcs)/2;
@@ -323,11 +280,9 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash, result;
     /* compare the integer vectors as generic memory areas. */
     if (memcmp(t, c, narcs * sizeof(int))) {
 
-      PROTECT(result = allocVector(STRSXP, 1));
-      SET_STRING_ELT(result, 0, mkChar("Different arc sets"));
-      UNPROTECT(3);
+      UNPROTECT(2);
 
-      return result;
+      return mkString("Different arc sets");
 
     }/*THEN*/
 
@@ -336,10 +291,7 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash, result;
   }/*THEN*/
 
   /* all checks completed successfully, returning TRUE. */
-  PROTECT(result = allocVector(LGLSXP, 1));
-  LOGICAL(result)[0] = TRUE;
-  UNPROTECT(1);
-
-  return result;
+  return ScalarLogical(TRUE);
 
 }/*ALL_EQUAL*/
+
