@@ -1,16 +1,13 @@
 #include "common.h"
 
-static double c_jt_stat(int **n, int *ni, int *llx, int *lly);
+static double c_jt_stat(int **n, int *ni, int llx, int lly);
 
-/* unconditional Jonckheere-Terpstra asymptotic test. */
-SEXP jt(SEXP x, SEXP y) {
+/* unconditional Jonckheere-Terpstra asymptotic test for use in C code. */
+double c_jt(int *xx, int llx, int *yy, int lly, int num) {
 
 int i = 0, j = 0, k = 0;
 int  **n = NULL, *ni = NULL, *nj = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y);
 double stat = 0, mean = 0, var = 0;
-SEXP result;
 
   /* initialize the contingency table and the marginal frequencies. */
   n = alloc2dcont(llx, lly);
@@ -18,12 +15,9 @@ SEXP result;
   nj = alloc1dcont(lly);
 
   /* compute the joint frequency of x and y. */
-  for (k = 0; k < num; k++) {
-
+  for (k = 0; k < num; k++) 
     n[xx[k] - 1][yy[k] - 1]++;
 
-  }/*FOR*/
-
   /* compute the marginals. */
   for (i = 0; i < llx; i++)
     for (j = 0; j < lly; j++) {
@@ -34,31 +28,23 @@ SEXP result;
   }/*FOR*/
 
   /* compute the test statistic, its mean and it variance. */
-  stat = c_jt_stat(n, ni, &llx, &lly);
-  mean = c_jt_mean(&num, ni, &llx);
-  var = c_jt_var(&num, ni, &llx, nj, &lly);
+  stat = c_jt_stat(n, ni, llx, lly);
+  mean = c_jt_mean(num, ni, llx);
+  var = c_jt_var(num, ni, llx, nj, lly);
 
   /* standardize before returning so to make the test invariant to
    * the order of the variables; if the variance is zero return zero
    * to imply independence. */
-  PROTECT(result = allocVector(REALSXP, 1));
-  NUM(result) = (var < MACHINE_TOL) ? 0 : (stat - mean) /sqrt(var);
-  UNPROTECT(1);
+  return (var < MACHINE_TOL) ? 0 : (stat - mean) / sqrt(var);
 
-  return result;
+}/*C_JT*/
 
-}/*JT*/
-
-/* conditional Jonckheere-Terpstra asymptotic test. */
-SEXP cjt(SEXP x, SEXP y, SEXP z) {
+/* conditional Jonckheere-Terpstra asymptotic test for use in C code. */
+double c_cjt(int *xx, int llx, int *yy, int lly, int *zz, int llz, int num) {
 
 int i = 0, j = 0, k = 0;
-int llx = NLEVELS(x), lly = NLEVELS(y), llz = NLEVELS(z);
-int num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y), *zz = INTEGER(z);
 int  ***n = NULL, **nrowt = NULL, **ncolt = NULL, *ncond = NULL;
 double stat = 0, var = 0, tvar = 0;
-SEXP result;
 
   /* initialize the contingency table. */
   n = alloc3dcont(llz, llx, lly);
@@ -89,30 +75,26 @@ SEXP result;
     /* this one is never observed, skip. */
     if (ncond[k] == 0) continue;
 
-    stat += c_jt_stat(n[k], nrowt[k], &llx, &lly) -
-              c_jt_mean(&(ncond[k]), nrowt[k], &llx);
-    var = c_jt_var(&(ncond[k]), nrowt[k], &llx, ncolt[k], &lly);
+    stat += c_jt_stat(n[k], nrowt[k], llx, lly) -
+              c_jt_mean(ncond[k], nrowt[k], llx);
+    var = c_jt_var(ncond[k], nrowt[k], llx, ncolt[k], lly);
     if (!ISNAN(var) && (var > MACHINE_TOL))
       tvar += var;
 
   }/*FOR*/
 
   /* guard against divide-by-zero errors and standardize the result. */
-  PROTECT(result = allocVector(REALSXP, 1));
-  NUM(result) = (tvar < MACHINE_TOL) ? 0 : stat / sqrt(tvar);
-  UNPROTECT(1);
+  return (tvar < MACHINE_TOL) ? 0 : stat / sqrt(tvar);
 
-  return result;
-
-}/*CJT*/
+}/*C_CJT*/
 
 /* asymptotic expected value of the Jonkheere-Terpstra test statistic. */
-double c_jt_mean(int *num, int *ni, int *llx) {
+double c_jt_mean(int num, int *ni, int llx) {
 
 int i = 0;
-double res = (double)(*num) * (*num);
+double res = (double)(num * num);
 
-  for (i = 0; i < *llx; i++)
+  for (i = 0; i < llx; i++)
     res -= ni[i] * ni[i];
 
   return res / 4;
@@ -120,37 +102,37 @@ double res = (double)(*num) * (*num);
 }/*C_JT_MEAN*/
 
 /* asymptotic variance of the Jonkheere-Terpstra test statistic. */
-double c_jt_var(int *num, int *ni, int *llx, int *nj, int *lly) {
+double c_jt_var(int num, int *ni, int llx, int *nj, int lly) {
 
 int i = 0, j = 0;
 double U1 = 0, U2a = 0, U2b = 0, U2 = 0, U3a = 0, U3b = 0, U3 = 0;
 double res = 0, t1 = 0, t2 = 0, t3 = 0;
 
   /* numerator, first term. */
-  U1 = (double)(*num) * (double)(*num - 1) * (double)(2 * (*num) + 5);
-  for (i = 0; i < *llx; i++)
+  U1 = (double)(num) * (double)(num - 1) * (double)(2 * num + 5);
+  for (i = 0; i < llx; i++)
     U1 -= (double)(ni[i]) * (double)(ni[i] -1) * (double)(2 * ni[i] + 5);
-  for (j = 0; j < *lly; j++)
+  for (j = 0; j < lly; j++)
     U1 -= (double)(nj[j]) * (double)(nj[j] -1) * (double)(2 * nj[j] + 5);
 
   /* numerator, second term. */
-  for (i = 0; i < *llx; i++)
+  for (i = 0; i < llx; i++)
     U2a += (double)(ni[i]) * (double)(ni[i]  - 1) * (double)(ni[i] - 2);
-  for (j = 0; j < *lly; j++)
+  for (j = 0; j < lly; j++)
     U2b += (double)(nj[j]) * (double)(nj[j]  - 1) * (double)(nj[j] - 2);
   U2 = U2a * U2b;
 
   /* numerator, third term. */
-  for (i = 0; i < *llx; i++)
+  for (i = 0; i < llx; i++)
     U3a += (double)(ni[i]) * (double)(ni[i]  - 1);
-  for (j = 0; j < *lly; j++)
+  for (j = 0; j < lly; j++)
     U3b += (double)(nj[j]) * (double)(nj[j]  - 1);
   U3 = U3a * U3b;
 
   /* terms in the denominators. */
   t1 = 72;
-  t2 = 36 * (double)(*num) * (double)((*num) - 1) * (double)((*num) - 2);
-  t3 = 8 * (double)(*num) * (double)((*num) - 1);
+  t2 = 36 * (double)(num) * (double)(num - 1) * (double)(num - 2);
+  t3 = 8 * (double)(num) * (double)(num - 1);
 
   res = U1/t1 + U2/t2 + U3/t3;
 
@@ -159,18 +141,18 @@ double res = 0, t1 = 0, t2 = 0, t3 = 0;
 }/*C_JT_VAR*/
 
 /* unconditional Jonckheere-Terpstra test statistic. */
-static double c_jt_stat(int **n, int *ni, int *llx, int *lly) {
+static double c_jt_stat(int **n, int *ni, int llx, int lly) {
 
 int i = 0, j = 0, s = 0, t = 0;
 double res = 0, ni2 = 0, wi = 0, w = 0;
 
-  for (i = 1; i < *llx; i++) {
+  for (i = 1; i < llx; i++) {
 
     ni2 = (double)(ni[i]) * ((double)(ni[i]) + 1)/2;
 
     for (j = 0; j < i; j++) {
 
-      for (s = 0, w = 0; s < *lly; s++) {
+      for (s = 0, w = 0; s < lly; s++) {
 
         for (t = 0, wi = 0; t < s; t++)
           wi += n[i][t] + n[j][t];
