@@ -1,19 +1,25 @@
 
 #include "common.h"
 
-SEXP x2 (SEXP x, SEXP y) {
+double c_x2(int *xx, int llx, int *yy, int lly, int num, double *df,
+    int adj) {
 
 int i = 0, j = 0, k = 0, **n = NULL, *ni = NULL, *nj = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y);
-double *res = NULL;
-SEXP result;
+double res = 0;
 
-  /* allocate and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 2));
-  res = REAL(result);
-  res[0] = 0;
-  res[1] = (double)(llx - 1) * (double)(lly - 1);
+  if (adj) {
+
+    /* if there are less than 5 observations per cell on average, assume the
+     * test does not have enough power and return independence. */
+    if (num < 5 * llx * lly) {
+
+      if (df) *df = 1;
+
+      return 0;
+
+    }/*THEN*/
+
+  }/*THEN*/
 
   /* initialize the contingency table and the marginal frequencies. */
   n = alloc2dcont(llx, lly);
@@ -21,12 +27,9 @@ SEXP result;
   nj = alloc1dcont(lly);
 
   /* compute the joint frequency of x and y. */
-  for (k = 0; k < num; k++) {
-
+  for (k = 0; k < num; k++)
     n[xx[k] - 1][yy[k] - 1]++;
 
-  }/*FOR*/
-
   /* compute the marginals. */
   for (i = 0; i < llx; i++)
     for (j = 0; j < lly; j++) {
@@ -41,31 +44,39 @@ SEXP result;
     for (j = 0; j < lly; j++) {
 
       if (n[i][j] != 0)
-        res[0] += (n[i][j] - ni[i] * (double)nj[j] / num) *
-                  (n[i][j] - ni[i] * (double)nj[j] / num) /
-                  (ni[i] * (double)nj[j] / num);
+        res += (n[i][j] - ni[i] * (double)nj[j] / num) *
+               (n[i][j] - ni[i] * (double)nj[j] / num) /
+               (ni[i] * (double)nj[j] / num);
 
     }/*FOR*/
 
-  UNPROTECT(1);
+  /* compute the degrees of freedom. */
+  if (df)
+    *df = adj ? df_adjust(ni, llx, nj, lly) : (llx - 1) * (lly - 1);
 
-  return result;
+  return res;
 
-}/*X2*/
+}/*C_X2*/
 
-SEXP cx2 (SEXP x, SEXP y, SEXP z) {
+double c_cx2(int *xx, int llx, int *yy, int lly, int *zz, int llz, int num,
+    double *df, int adj) {
 
 int i = 0, j = 0, k = 0, ***n = NULL, **ni = NULL, **nj = NULL, *nk = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), llz = NLEVELS(z), num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y), *zz = INTEGER(z);
-double *res = NULL;
-SEXP result;
+double res = 0;
 
-  /* allocate  and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 2));
-  res = REAL(result);
-  res[0] = 0;
-  res[1] = (double)(llx - 1) * (double)(lly - 1) * (double)llz;
+  if (adj) {
+
+    /* if there are less than 5 observations per cell on average, asuume the
+     * test does not have enough power and return independence. */
+    if (num < 5 * llx * lly * llz) {
+
+      if (df) *df = 1;
+
+      return 0;
+
+    }/*THEN*/
+
+  }/*THEN*/
 
   /* initialize the contingency table and the marginal frequencies. */
   n = alloc3dcont(llx, lly, llz);
@@ -74,12 +85,9 @@ SEXP result;
   nk = alloc1dcont(llz);
 
   /* compute the joint frequency of x, y, and z. */
-  for (k = 0; k < num; k++) {
-
+  for (k = 0; k < num; k++)
     n[xx[k] - 1][yy[k] - 1][zz[k] - 1]++;
 
-  }/*FOR*/
-
   /* compute the marginals. */
   for (i = 0; i < llx; i++)
     for (j = 0; j < lly; j++)
@@ -97,15 +105,17 @@ SEXP result;
       for (k = 0; k < llz; k++) {
 
         if (n[i][j][k] != 0)
-          res[0] += (n[i][j][k] - ni[i][k] * (double)nj[j][k] / nk[k]) *
-                    (n[i][j][k] - ni[i][k] * (double)nj[j][k] / nk[k]) /
-                    (ni[i][k] * (double)nj[j][k] / nk[k]);
+          res += (n[i][j][k] - ni[i][k] * (double)nj[j][k] / nk[k]) *
+                 (n[i][j][k] - ni[i][k] * (double)nj[j][k] / nk[k]) /
+                 (ni[i][k] * (double)nj[j][k] / nk[k]);
 
       }/*FOR*/
 
-  UNPROTECT(1);
+  /* compute the degrees of freedom. */
+  if (df)
+    *df = adj ? cdf_adjust(ni, llx, nj, lly, llz) : (llx - 1) * (lly - 1) * llz;
 
-  return result;
+  return res;
 
-}/*CX2*/
+}/*C_CX2*/
 
