@@ -1,25 +1,16 @@
 
 #include "common.h"
 
-static void _mi_lambda(double *n, double *lambda, double *target, int *num,
-    int *llx, int *lly, int *llz);
+static void _mi_lambda(double *n, double *lambda, double target, int num,
+    int llx, int lly, int llz);
 
-/* shrinked mutual information, to be used for the asymptotic test. */
-SEXP shmi(SEXP x, SEXP y, SEXP gsquare) {
+/* shrinked mutual information, to be used in C code. */
+double c_shmi(int *xx, int llx, int *yy, int lly, int num) {
 
 int i = 0, j = 0, k = 0;
 double **n = NULL, *ni = NULL, *nj = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y);
 double lambda = 0, target = 1/(double)(llx * lly);
-double *res = NULL;
-SEXP result;
-
-  /* allocate and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 2));
-  res = REAL(result);
-  res[0] = 0;
-  res[1] = (double)(llx - 1) * (double)(lly - 1);
+double res = 0;
 
   /* initialize the contingency table and the marginal frequencies. */
   n = alloc2dreal(llx, lly);
@@ -27,14 +18,11 @@ SEXP result;
   nj = alloc1dreal(lly);
 
   /* compute the joint frequency of x and y. */
-  for (k = 0; k < num; k++) {
-
+  for (k = 0; k < num; k++)
     n[xx[k] - 1][yy[k] - 1]++;
 
-  }/*FOR*/
-
   /* estimate the optimal lambda for the data. */
-  _mi_lambda((double *)n, &lambda, &target, &num, &llx, &lly, NULL);
+  _mi_lambda((double *)n, &lambda, target, num, llx, lly, 0);
 
   /* switch to the probability scale and shrink the estimates. */
   for (i = 0; i < llx; i++)
@@ -52,41 +40,25 @@ SEXP result;
 
   /* compute the mutual information from the joint and marginal frequencies. */
   for (i = 0; i < llx; i++)
-    for (j = 0; j < lly; j++) {
-
+    for (j = 0; j < lly; j++)
       if (n[i][j] != 0)
-        res[0] += n[i][j] * log(n[i][j] / (ni[i] * nj[j]));
+        res += n[i][j] * log(n[i][j] / (ni[i] * nj[j]));
 
-    }/*FOR*/
+  return res;
 
-  /* rescale to match the G^2 test. */
-  if (isTRUE(gsquare))
-    res[0] *= 2 * num;
+}/*C_SHMI*/
 
-  UNPROTECT(1);
-
-  return result;
-
-}/*SHMI*/
-
-/* shrinked conditional mutual information, to be used for the asymptotic
- * test. */
-SEXP shcmi(SEXP x, SEXP y, SEXP z, SEXP gsquare) {
+/* shrinked conditional mutual information, to be used in C code. */
+double c_shcmi(int *xx, int llx, int *yy, int lly, int *zz, int llz,
+    int num, double *df) {
 
 int i = 0, j = 0, k = 0;
 double ***n = NULL, **ni = NULL, **nj = NULL, *nk = NULL;
-int llx = NLEVELS(x), lly = NLEVELS(y), llz = NLEVELS(z);
-int num = length(x);
-int *xx = INTEGER(x), *yy = INTEGER(y), *zz = INTEGER(z);
 double lambda = 0, target = 1/(double)(llx * lly * llz);
-double *res = NULL;
-SEXP result;
+double res = 0;
 
-  /* allocate and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 2));
-  res = REAL(result);
-  res[0] = 0;
-  res[1] = (double)(llx - 1) * (double)(lly - 1) * (double)llz;
+  /* compute the degrees of freedom. */
+  *df = (double)(llx - 1) * (double)(lly - 1) * (double)(llz);
 
   /* initialize the contingency table and the marginal frequencies. */
   n = alloc3dreal(llx, lly, llz);
@@ -95,14 +67,11 @@ SEXP result;
   nk = alloc1dreal(llz);
 
   /* compute the joint frequency of x, y, and z. */
-  for (k = 0; k < num; k++) {
-
+  for (k = 0; k < num; k++)
     n[xx[k] - 1][yy[k] - 1][zz[k] - 1]++;
 
-  }/*FOR*/
-
   /* estimate the optimal lambda for the data. */
-  _mi_lambda((double *)n, &lambda, &target, &num, &llx, &lly, &llz);
+  _mi_lambda((double *)n, &lambda, target, num, llx, lly, llz);
 
   /* switch to the probability scale and shrink the estimates. */
   for (i = 0; i < llx; i++)
@@ -133,7 +102,7 @@ SEXP result;
       for (i = 0; i < llx; i++) {
 
         if (n[i][j][k] > 0)
-          res[0] += n[i][j][k] * log( (n[i][j][k] * nk[k]) / (ni[i][k] * nj[j][k]) );
+          res += n[i][j][k] * log( (n[i][j][k] * nk[k]) / (ni[i][k] * nj[j][k]) );
 
       }/*FOR*/
 
@@ -141,32 +110,26 @@ SEXP result;
 
   }/*FOR*/
 
-  /* rescale to match the G^2 test. */
-  if (isTRUE(gsquare))
-    res[0] *= 2 * num;
+  return res;
 
-  UNPROTECT(1);
-
-  return result;
-
-}/*SHCMI*/
+}/*C_SHCMI*/
 
 /* compute the shrinkage intensity lambda for the mutual information. */
-static void _mi_lambda(double *n, double *lambda, double *target, int *num,
-    int *llx, int *lly, int *llz) {
+static void _mi_lambda(double *n, double *lambda, double target, int num,
+    int llx, int lly, int llz) {
 
 double lden = 0, lnum = 0, temp = 0;
 
   /* compute the numerator and the denominator of the shrinkage intensity;
    * if the third dimension is a NULL pointer it's a 2-dimensional table. */
-  if (!llz) {
+  if (llz == 0) {
 
-    for (int i = 0; i < *llx; i++)
-      for (int j = 0; j < *lly; j++) {
+    for (int i = 0; i < llx; i++)
+      for (int j = 0; j < lly; j++) {
 
-        temp = ((double **)n)[i][j] / (double)(*num);
+        temp = ((double **)n)[i][j] / (double)(num);
         lnum += temp * temp;
-        temp = *target - ((double **)n)[i][j] / (double)(*num);
+        temp = target - ((double **)n)[i][j] / (double)(num);
         lden += temp * temp;
 
       }/*FOR*/
@@ -174,13 +137,13 @@ double lden = 0, lnum = 0, temp = 0;
   }/*THEN*/
   else {
 
-    for (int i = 0; i < *llx; i++)
-      for (int j = 0; j < *lly; j++)
-        for (int k = 0; k < *llz; k++) {
+    for (int i = 0; i < llx; i++)
+      for (int j = 0; j < lly; j++)
+        for (int k = 0; k < llz; k++) {
 
-          temp = ((double ***)n)[i][j][k] / (double)(*num);
+          temp = ((double ***)n)[i][j][k] / (double)(num);
           lnum += temp * temp;
-          temp = *target - ((double ***)n)[i][j][k] / (double)(*num);
+          temp = target - ((double ***)n)[i][j][k] / (double)(num);
           lden += temp * temp;
 
       }/*FOR*/
@@ -191,7 +154,7 @@ double lden = 0, lnum = 0, temp = 0;
   if (lden == 0)
     *lambda = 1;
   else
-    *lambda = (1 - lnum) / ((double)(*num - 1) * lden);
+    *lambda = (1 - lnum) / ((double)(num - 1) * lden);
 
   /* bound the shrinkage intensity in the [0,1] interval. */
   if (*lambda > 1)
@@ -201,18 +164,13 @@ double lden = 0, lnum = 0, temp = 0;
 
 }/*_MI_LAMBDA*/
 
-/* Shrinked Linear Correlation. */
-SEXP fast_shcor(SEXP x, SEXP y, SEXP length) {
+/* shrinked linear correlation, to be used in C code. */
+double c_fast_shcor(double *xx, double *yy, int *n) {
 
-int i = 0, *n = INTEGER(length);
-double *sum = NULL, *xx = REAL(x), *yy = REAL(y);
+int i = 0;
+double sum = 0;
 double xm = 0, ym = 0, xsd = 0, ysd = 0, lambda = 0;
 double tol = MACHINE_TOL;
-SEXP res;
-
-  PROTECT(res = allocVector(REALSXP, 1));
-  sum = REAL(res);
-  *sum = 0;
 
   /* compute the mean values.  */
   for (i = 0 ; i < *n; i++) {
@@ -228,7 +186,7 @@ SEXP res;
   /* compute the actual covariance. */
   for (i = 0; i < *n; i++) {
 
-    *sum += (xx[i] - xm) * (yy[i] - ym);
+    sum += (xx[i] - xm) * (yy[i] - ym);
     xsd += (xx[i] - xm) * (xx[i] - xm);
     ysd += (yy[i] - ym) * (yy[i] - ym);
 
@@ -236,16 +194,16 @@ SEXP res;
 
   /* note that the shrinkage intesity for the correlation coefficient is
    * identical to the one for the covariance; so we don't need to standaridize
-   * the data*/
+   * the data. */
   for (i = 0; i < *n; i++) {
 
-    lambda += ((xx[i] - xm) * (yy[i] - ym) - (*sum)/(*n)) *
-              ((xx[i] - xm) * (yy[i] - ym) - (*sum)/(*n));
+    lambda += ((xx[i] - xm) * (yy[i] - ym) - sum / (*n)) *
+              ((xx[i] - xm) * (yy[i] - ym) - sum / (*n));
 
   }/*FOR*/
 
   /* compute lambda, the shrinkage intensity. */
-  lambda *= ((*n) / (*sum)) * ((*n) / (*sum));
+  lambda *= (*n / sum) * (*n / sum);
   lambda *= (double)(*n) / (double)(*n - 1) / (double)(*n -1) / (double)(*n -1);
 
   /* truncate the shrinkage intensity in the [0,1] interval; this is not an
@@ -263,63 +221,35 @@ SEXP res;
 
   /* safety check against "divide by zero" errors. */
   if (fabs(xsd) < tol || fabs(ysd) < tol)
-    *sum = 0;
+    sum = 0;
   else
-    *sum *= (1 - lambda) / (sqrt(xsd) * sqrt(ysd));
+    sum *= (1 - lambda) / (sqrt(xsd) * sqrt(ysd));
 
   /* double check that the coefficient is in the [-1, 1] range. */
-  if (*sum > 1) {
+  if (sum > 1) {
 
     warning("fixed correlation coefficient greater than 1, probably due to floating point errors.");
 
-    *sum = 1;
+    sum = 1;
 
   }/*THEN*/
-  else if (*sum < -1) {
+  else if (sum < -1) {
 
     warning("fixed correlation coefficient lesser than -1, probably due to floating point errors.");
 
-    *sum = -1;
+    sum = -1;
 
   }/*ELSE*/
 
-  UNPROTECT(1);
+  return sum;
 
-  return res;
-
-}/*FAST_SHCOR*/
+}/*C_FAST_SHCOR*/
 
 /* Shrinked Covariance Matrix. */
-SEXP cov_lambda(SEXP data, SEXP length) {
+void c_cov_lambda(double **column, double *mean, int ncols, int n, double *var) {
 
 int i = 0, j = 0, k = 0, cur = 0;
-int *n = INTEGER(length), ncols = length(data);
-double *mean = NULL, *var = NULL, **column = NULL;
 double lambda = 0, sumcors = 0, sumvars = 0;
-SEXP res;
-
-  /* allocate the covariance matrix. */
-  PROTECT(res = allocMatrix(REALSXP, ncols, ncols));
-  var = REAL(res);
-  memset(var, '\0', ncols * ncols * sizeof(double));
-
-  /* allocate an array to store the mean values. */
-  mean = alloc1dreal(ncols);
-
-  /* allocate and initialize an array of pointers for the variables. */
-  column = (double **) alloc1dpointer(ncols);
-  for (i = 0; i < ncols; i++)
-    column[i] = REAL(VECTOR_ELT(data, i));
-
-  /* compute the mean values  */
-  for (i = 0; i < ncols; i++) {
-
-    for (j = 0 ; j < *n; j++)
-      mean[i] += column[i][j];
-
-    mean[i] /= (*n);
-
-  }/*FOR*/
 
   for (i = 0; i < ncols; i++) {
 
@@ -328,26 +258,26 @@ SEXP res;
       cur = CMC(i, j, ncols);
 
       /* compute the actual variance/covariance. */
-      for (k = 0; k < *n; k++)
+      for (k = 0; k < n; k++)
         var[cur] += (column[i][k] - mean[i]) * (column[j][k] - mean[j]);
 
       if (i != j) {
 
         /* do the first round of computations for the shrinkage intensity. */
-        for (k = 0; k < *n; k++) {
+        for (k = 0; k < n; k++) {
 
           sumvars +=
-            ((column[i][k] - mean[i]) * (column[j][k] - mean[j]) - var[cur] / (*n)) *
-            ((column[i][k] - mean[i]) * (column[j][k] - mean[j]) - var[cur] / (*n));
+            ((column[i][k] - mean[i]) * (column[j][k] - mean[j]) - var[cur] / n) *
+            ((column[i][k] - mean[i]) * (column[j][k] - mean[j]) - var[cur] / n);
 
         }/*FOR*/
 
-        sumcors += (var[cur] / (*n - 1)) * (var[cur] / (*n - 1));
+        sumcors += (var[cur] / (n - 1)) * (var[cur] / (n - 1));
 
       }/*THEN*/
 
       /* use the unbiased estimator for variances/covariances. */
-      var[cur] /= (*n) - 1;
+      var[cur] /= n - 1;
 
       /* fill in the symmetric element of the matrix. */
       var[CMC(j, i, ncols)] = var[cur];
@@ -357,7 +287,7 @@ SEXP res;
   }/*FOR*/
 
   /* wrap up the computation of the shrinkage intensity. */
-  lambda = sumvars * (*n) / (*n - 1) / (*n -1) / (*n -1) / sumcors;
+  lambda = sumvars * n / (n - 1) / (n -1) / (n -1) / sumcors;
 
   /* truncate the shrinkage intensity in the [0,1] interval; this is not an
    * error, but a measure to increase the quality of the shrinked estimate. */
@@ -378,8 +308,5 @@ SEXP res;
       if (i != j)
         var[CMC(i, j, ncols)] *= 1 - lambda;
 
-  UNPROTECT(1);
-  return res;
-
-}/*COV_LAMBDA*/
+}/*C_COV_LAMBDA*/
 
